package visl.rasmus.resources;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

/**
* ResourceBundle-like class for encapsulation of the symbol set of a given 
* language.
* See the localization HOWTO on how to define the symbol sets for each 
* language.
* @author Martin Carlsen
* @author Rasmus Lock Larsen - added the depkernel functionality (Oct. 2000)
* @author Rasmus Lock Larsen - added the field functionality (Feb. 2001)
*/
public class GrammarSymbols
{
    /** Default Localization (empty symbol set). */
    public void localize() {
	String[][] defaultfunctions = {
	    {"?", "?"}
	};
	String[][] defaultforms = {
	    {"?", "?"}
	};
	String[][] defaultdepkernel = null;
	String[][] defaultfield = null;
	initTables(defaultfunctions, defaultforms, 
		   defaultdepkernel, defaultfield);
    }
	
    /**
     * Read symbol set from url.
     *@param funcUrl url to read function symbols from from.
     *@param formUrl url to read form symbols from from.
     *@param depkernelUrl url to read dep-kernel relations from.
     *@exception MalformedURLException if any of the URL
     */
    public static void localize(String funcUrl, String formUrl, 
				String depkernelUrl, String fieldUrl) throws 
				MalformedURLException, IOException{
	if (funcUrl != null) function = readSet(funcUrl);
	else function = null;
	if (formUrl != null) form = readSet(formUrl);
	else form = null;
	if (depkernelUrl != null) depkernel = readSet(depkernelUrl);
	else depkernel = null;
	if (fieldUrl != null) field = readSet(fieldUrl);
	else field = null;
	initTables(function, form, depkernel, field);
    }
	
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({ABBR}'='{FULL_NAME}'\n')*
     */
    public static String[][] readSet(String url) throws 
    MalformedURLException, IOException{
	return readSet(new URL(url));
    }
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({ABBR}'='{FULL_NAME}'\n')*
     */
    public static String[][] readSet(URL url) throws MalformedURLException, IOException {
	Vector set = new Vector();
	String s = null;
	//BufferedReader d = null;
	//d = new BufferedReader(new InputStreamReader(url.openStream())); 
	// use a BufferedReader for input.
	DataInputStream d = null;
	d = new DataInputStream(url.openStream());
	s = d.readLine(); // read the first line
	String abbr,full; // abbreviated and full name of symbol
	int equ; // position of the '=' character
	while(s!=null){ // while not EOF
	    equ = s.indexOf('=');
	    if(equ>=0 && !s.startsWith("#")){
		abbr = s.substring(0,equ);
		full = s.substring(equ+1,s.length());
		set.addElement(new String[]{abbr,full}); 
		// Add the (abbr,full name) pair to the set of symbols
	    }
	    s = d.readLine(); // read the next line.
	}
		
	int n = set.size(); // number of symbols
	String[][] symbols = new String[n][];
	for(int i=0; i<n; i++){
	    symbols[i] = (String[]) set.elementAt(i);
	}
	return symbols;
    }
	
    /**
     * {abbr., full form name} pairs for forms.
     */
    static String[][] form;
    /**
     * {abbr., full function name} pairs for functions.
     */
    static String[][] function;
    /**
     * {dep, kernel} pairs for dependent and kernel relations.
     */
    static String[][] depkernel;
    /**
     * {field-short, field-long} pairs for the fieldnotation.
     */
    static String[][] field;
    /**
     * The {abbr., full form name} pairs as a Hashtable.
     */
    static Hashtable formtable;
    /**
     * The {abbr., full function name} pairs as a Hashtable.
     */
    static Hashtable functiontable;	
    /**
     * The {dep, kernel} pairs as Hashtable.
     */
    static Hashtable depkerneltable;
    /**
     * The {short, long} pairs as Hashtable.
     */
    static Hashtable fieldtable;

    /**
     * Initialize tables. Called by the localize method.
     * @param functionset Array of {"abbr", "full function name"} pairs.
     * @param formset Array of {"abbr", "full form name"} pairs.
     * @param depkernelset Array of {"dep", "kernel"} pairs (abbr.).
     * @param fieldset Array of {short, long} pairs
     */
    static void initTables(String[][] functionset, String[][] formset,
			   String[][] depkernelset, String[][] fieldset) {
	function = functionset;
	form = formset;
	depkernel = depkernelset;
	field = fieldset;

	formtable = new Hashtable();
	functiontable = new Hashtable();
	depkerneltable = new Hashtable();
	fieldtable = new Hashtable();

	if (form != null) {
	    for (int i=0; i<form.length; i++) {
		formtable.put(form[i][0], form[i][1].trim());
	    }
	}
	if (function != null) {
	    for (int i=0; i<function.length; i++) {
		functiontable.put(function[i][0], function[i][1].trim());
	    }
	}
	if (depkernel != null)
	    for (int i=0; i<depkernel.length; i++) {
		depkerneltable.put(depkernel[i][0].trim(), 
				   depkernel[i][1].trim());
	    }
	if (field != null)
	    for (int i = 0; i < field.length; i++) {
		fieldtable.put(field[i][0].trim(), 
			       field[i][1].trim());
	    }
    }	
    
    /**
     * Get the number of forms.
     *@return The number of form symbols used by the language.
     */
    public static int countForms(){
	return form.length;
    }
	
    /**
     * Get the number of functions.
     *@return The number of function symbols used by the language.	
     */
    public static int countFunctions(){
	return function.length;
    }
    public static int countFields() {
	return field.length;
    }
    //
    // Getters (by number) for full and abbreviated form and function names.
    //
	
    /**
     * Get an abbreviated form name.
     *@param number Number of the form to get.
     *@return The abbreviation of form number number.
     */
    public static String getFormAbbr(int number){
	String s = "?"; // Default (= unknown)
	try{
	    s = form[number][0];
	}
	catch(ArrayIndexOutOfBoundsException e){}
	return s;
    }
	
    /**
     * Get an abbreviated function name.
     *@param number Number of the function to get.
     *@return The abbreviation of function number number.
     */
    public static String getFunctionAbbr(int number){
	String s = "?"; // Default (= unknown)
	try{
	    s = function[number][0];
	}
	catch(ArrayIndexOutOfBoundsException e){}
	return s;
    }
    public static String getFieldAbbr(int number) {
	String s = "?"; // Default (= unknown)
	try {
	    s = field[number][0];
	}
	catch (ArrayIndexOutOfBoundsException e) {}
	return s;
    }
	
    /**
     * Get the full name of a form.
     *@param number Number of the form to get.
     *@return The full name of form number number.
     */
    public static String getForm(int number){
	String s = (String) formtable.get("?"); // Default (= unknown)
	try{
	    s = form[number][1];
	}
	catch(ArrayIndexOutOfBoundsException e){}
	return s;
    }
	
    /**
     * Get the full name of a function.
     *@param number Number of the function to get.
     *@return The full name of function number number.
     */
    public static String getFunction(int number){
	String s = (String) functiontable.get("?"); // Default (= unknown)
	try{
	    s = function[number][1];
	}
	catch(ArrayIndexOutOfBoundsException e){}
	return s;
    }
	
	
    //
    // Conversion between full and abbreviated function and form names
    //
	
    /**
     * Abbreviate form name.
     *@param fullname The full name of the form.
     *@return The abbreviation of the form.
     */
    public static String getFormAbbr(String fullname){
	// this is not very elegent.
	for(int i=0; i<countForms(); i++){ // Search for the form.
	    if( ((String)getForm(i).trim()).equals(fullname.trim())) return getFormAbbr(i); // form found
	}
	return "?"; // Default (= unknown)
    }
	
    /**
     * Abbreviate function name.
     *@param fullname The full name of the function.
     *@return The abbreviation of the function.
     */
    public static String getFunctionAbbr(String fullname){
	// this is not very elegant.
	for(int i=0; i<countFunctions(); i++){ // Search for the function
	    if( ((String)getFunction(i).trim()).equals(fullname.trim())) return getFunctionAbbr(i);
	}
	return "?"; // Default (= unknown)
    }
    /**
     * Get full form name from abbreviation.
     *@param abbrname The abbreviated form name.
     *@return The full name of the form.
     */
    public static String getForm(String abbrname){
	String s = (String) formtable.get(abbrname);
	if (s == null) s = (String) formtable.get("?"); // Default (= unknown)
	if (s == null) return s;
	return s.trim();
    }
	
    /**
     * Get full function name from abbreviation.
     *@param abbrname The abbreviated function name.
     *@return The full name of the function.
     */
    public static String getFunction(String abbrname){
	String s = (String) functiontable.get(abbrname);
	// Default (= unknown)
	if (s == null) s = (String) functiontable.get("?");
	if (s == null) return s;
	return s.trim();
    }
    public static String getField(String abbrname) {
	String s = (String) fieldtable.get(abbrname);
	// Default (= unknown)
	if (s == null) s = (String) fieldtable.get("?"); 
	return s.trim();
    }
    //
    // Extraction methods for dep-kernel
    //
    public static String getKernel(String abbrdep) {
	String s = (String) depkerneltable.get(abbrdep);
	return s;
    }
}



